from typing import Union

from finstruments.common.base import Base
from finstruments.common.decorators import serializable
from finstruments.common.enum import Currency
from finstruments.instrument.cryptocurrency import Cryptocurrency


@serializable
class CurrencyPair(Base):
    """
    Represents a trading pair between two assets, which can be cryptocurrencies or fiat currencies.
    """

    base_currency: Union[Cryptocurrency, Currency]
    quote_currency: Union[Cryptocurrency, Currency]

    def __str__(self):
        base_currency = (
            self.base_currency.value
            if isinstance(self.base_currency, Currency)
            else self.base_currency.ticker
        )
        quote_currency = (
            self.quote_currency.value
            if isinstance(self.quote_currency, Currency)
            else self.quote_currency.ticker
        )
        return f"{base_currency}/{quote_currency}"
