from abc import ABC
from datetime import date
from typing import Optional

from pydantic import Field

from finstruments.common.decorators import serializable, serializable_base_class
from finstruments.common.enum import Currency
from finstruments.instrument.abstract import BaseInstrument


@serializable_base_class
class BaseCryptocurrency(BaseInstrument, ABC):
    """
    Cryptocurrency base class.
    """

    ticker: str
    full_name: Optional[str] = Field(default=None)
    network: Optional[str] = Field(default=None)
    contract_address: Optional[str] = Field(default=None)
    agreed_discount_rate: Optional[str] = Field(init=False, default=None)
    pillar_date: Optional[date] = Field(init=False, default=None)
    denomination_currency: Optional[Currency] = Field(init=False, default=None)
    code: str


@serializable
class Cryptocurrency(BaseCryptocurrency):
    """
    Cryptocurrency.
    """

    code: str = Field(init=False, default="CRYPTOCURRENCY")
