import unittest

from finstruments.common.enum import Currency
from finstruments.instrument.common.currency_pair import CurrencyPair
from finstruments.instrument.cryptocurrency import Cryptocurrency
from tests.unit.deserialization.util import assert_serialization


class TestCurrencyPairDeserialization(unittest.TestCase):
    def test_cryptocurrency_pair(self):
        sol = Cryptocurrency(
            ticker="SOL", full_name="Solana", network="Solana", contract_address=None
        )
        bingus = Cryptocurrency(
            ticker="BINGUS",
            full_name="Bingus the Cat",
            network="Solana",
            contract_address="AQuuQ4xktyzGBFnbKHnYsXHxsKVQetAoiPeCEG97NUJw",
        )
        sol_bingus = CurrencyPair(base_currency=sol, quote_currency=bingus)

        assert_serialization(sol_bingus, CurrencyPair)

    def test_currency_pair(self):
        eur_usd = CurrencyPair(base_currency=Currency.EUR, quote_currency=Currency.USD)

        assert_serialization(eur_usd, CurrencyPair)

    def test_mixed_currency_pair(self):
        btc = Cryptocurrency(
            ticker="BTC", full_name="Bitcoin", network="Bitcoin", contract_address=None
        )
        btc_usd = CurrencyPair(
            base_currency=btc,
            quote_currency=Currency.USD,
        )

        assert_serialization(btc_usd, CurrencyPair)
